
/*
  A simple Pong game:
 */

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define UP_BUTTON 2
#define DOWN_BUTTON 3
#define BUZZER 5
volatile int up = 0;
volatile int down = 0;

const unsigned long PADDLE_RATE = 33;
const unsigned long BALL_RATE = 16;
const uint8_t PADDLE_HEIGHT = 24;

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

void drawCourt();
void resetGame();

uint8_t ball_x = 64, ball_y = 32;
uint8_t ball_dir_x = 1, ball_dir_y = 1;
unsigned long ball_update;

unsigned long paddle_update;
const uint8_t CPU_X = 12;
uint8_t cpu_y = 16;

const uint8_t PLAYER_X = 115;
uint8_t player_y = 16;

uint8_t player_score = 0;
uint8_t cpu_score = 0;

void setup_interrupts() {


  cli();
 EICRA |= (1 << ISC00) | (1 << ISC10);
 EIMSK |= (1 << INT0);
 EIMSK |= (1 << INT1);
  // input
  DDRD &= ~(1 << PD2) & ~(1 << PD3);
  // input pullup
  PORTD |= (1 << PD2) | (1 << PD3);
  sei();
}
void setup() {

    Serial.begin(9600);
    display.begin(SSD1306_SWITCHCAPVCC, 0x3C);

    display.display();
    unsigned long start = millis();

    setup_interrupts();
    display.clearDisplay();
    drawCourt();

    while (millis() - start < 2000);

    display.display();

    ball_update = millis();
    paddle_update = ball_update;
}
ISR(INT0_vect)
{
  //PD2
  if((PIND &(1<<PD2)) == 0){

    //este apasat
    up = 1;

  }
  else{
    up = 0;
  }
}
ISR(INT1_vect)
{
  //PD3
  if((PIND &(1<<PD3)) == 0){

    //este apasat
    down = 1;

  }
  else{
    down = 0;
  }
}

void loop() {
    bool update = false;
    unsigned long time = millis();

    static bool up_state = false;
    static bool down_state = false;

    up_state |= ( up == LOW);
    down_state |= (down== LOW);

    if (time > ball_update) {
        uint8_t new_x = ball_x + ball_dir_x;
        uint8_t new_y = ball_y + ball_dir_y;

        // Check if we hit the vertical walls
        if (new_x == 0 || new_x == 127) {
            tone(BUZZER, 150, 100);
            ball_dir_x = -ball_dir_x;
            new_x += ball_dir_x + ball_dir_x;
        }

        // Check if we hit the horizontal walls.
        if (new_y == 0 || new_y == 63) {
            tone(BUZZER, 150, 100);
            ball_dir_y = -ball_dir_y;
            new_y += ball_dir_y + ball_dir_y;
        }

        // Check if we hit the CPU paddle
        if (new_x == CPU_X && new_y >= cpu_y && new_y <= cpu_y + PADDLE_HEIGHT) {
            tone(BUZZER, 400, 200);
            ball_dir_x = -ball_dir_x;
            new_x += ball_dir_x + ball_dir_x;
        }

        // Check if we hit the player paddle
        if (new_x == PLAYER_X && new_y >= player_y && new_y <= player_y + PADDLE_HEIGHT) {
            tone(BUZZER, 400, 200);
            ball_dir_x = -ball_dir_x;
            new_x += ball_dir_x + ball_dir_x;
        }

        display.drawPixel(ball_x, ball_y, BLACK);
        display.drawPixel(new_x, new_y, WHITE);
        ball_x = new_x;
        ball_y = new_y;

        ball_update += BALL_RATE;

        update = true;
    }

    if (time > paddle_update) {
        paddle_update += PADDLE_RATE;

        // CPU paddle
        display.drawFastVLine(CPU_X, cpu_y, PADDLE_HEIGHT, BLACK);
        const uint8_t half_paddle = PADDLE_HEIGHT >> 1;
        if (cpu_y + half_paddle > ball_y) {
            cpu_y -= 1;
        }
        if (cpu_y + half_paddle < ball_y) {
            cpu_y += 1;
        }
        if (cpu_y < 1) cpu_y = 1;
        if (cpu_y + PADDLE_HEIGHT > 63) cpu_y = 63 - PADDLE_HEIGHT;
        display.drawFastVLine(CPU_X, cpu_y, PADDLE_HEIGHT, WHITE);

        // Player paddle
        display.drawFastVLine(PLAYER_X, player_y, PADDLE_HEIGHT, BLACK);
        if (up_state) {
            player_y -= 1;
        }
        if (down_state) {
            player_y += 1;
        }
        up_state = down_state = false;
        if (player_y < 1) player_y = 1;
        if (player_y + PADDLE_HEIGHT > 63) player_y = 63 - PADDLE_HEIGHT;
        display.drawFastVLine(PLAYER_X, player_y, PADDLE_HEIGHT, WHITE);

        update = true;
    }

    if (update)
        display.display();

    // Check if a player scored
    //Serial.println(ball_x);
    if (ball_x < 13) {
        player_score++;
        resetGame();
    } else if (ball_x >= 126) {
        cpu_score++;
        resetGame();
    }

    // Check if a player reached 5 points
    if (player_score == 5 || cpu_score == 5) {
        display.clearDisplay();
        display.setTextSize(2);
        display.setTextColor(WHITE);
        display.setCursor(25, 20);
        if (player_score == 5) {
            tone(BUZZER, 400, 200);
            // display.println("Player Wins!");
            //display.println("player VS bot");
            display.print(player_score);
            display.print("  ");
            display.print(cpu_score);
        } else {
            tone(BUZZER, 400, 200);
            // display.println("CPU Wins!");
            //display.println("player VS bot");
            display.print(player_score);
            display.print("    ");
            display.print(cpu_score);
        }
        display.display();
        while (true); // Stop the game
    }
}

void resetGame() {
    ball_x = 64;
    ball_y = 32;
    ball_dir_x = random(2) == 0 ? -1 : 1;
    ball_dir_y = random(2) == 0 ? -1 : 1;
    cpu_y = 16;
    player_y = 16;

    display.clearDisplay();
    drawCourt();
    display.display();
    delay(2000);

    ball_update = millis();
    paddle_update = ball_update;
}

void drawCourt() {
    display.drawRect(0, 0, 128, 64, WHITE);
}
